# frozen_string_literal: true

module Hermes
  class BorderingTown
    include Hermes::Concerns::CommonFieldsConcern
    
    field :insee_code, type: String
    field :name, type: String
    field :bordering_towns, type: Array, default: []
    field :last_update_from_csv_at, type: DateTime
    
    validates_presence_of :insee_code
    validates_uniqueness_of :insee_code
    
    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    common_indexes *%i[
      insee_code
      name
      bordering_towns
      last_update_from_csv_at
    ]

    def self.import_from_csv(data)
      table = CSV.parse(data, headers: true, col_sep:  "," )
      count = table.length  

      hash_csv = {}

      table.each do |row|
        hash_csv[row["insee"]] = { 
          bordering_towns: (row["insee_voisins"].split('|') + ((hash_csv[row["insee"]] && hash_csv[row["insee"]][:bordering_towns]) || [])).uniq, 
          name: row["nom"]
        }
      end 
    
      hash_csv.each do |insee, row|
        bt = Hermes::BorderingTown.find_or_create_by(insee_code: insee.to_s)
        bt.bordering_towns = row[:bordering_towns]
        if bt.bordering_towns_changed?
          bt.set(updated_at: DateTime.current, bordering_towns: row[:bordering_towns])
          Hermes::HousingRequest.where(is_active: true, seeked_district_insee_codes: bt.insee_code).batch_size(5).no_timeout.each do |hr|
            begin
              hr.update_bordering_insee_codes
              hr.update_eligibility_query
              hr.set(
                updated_at: DateTime.current,
                bordering_insee_codes: hr.bordering_insee_codes,
                eligibility_query_bordering: hr.eligibility_query_bordering,
                eligibility_query_seeked_bordering: hr.eligibility_query_seeked_bordering,
              )
            rescue e
              Hermes::LogJob.add_to_queue(level: 'error', code: "BORDERING_TOWNS_REQUEST_UPDATE", action: "Error when updating request #{hr.id}", data: e.as_json)
            end
          end 
        end 

        bt.update!(
          name: row[:name],
          last_update_from_csv_at: DateTime.current,
        )
      end 
    end

  end
end
