# frozen_string_literal: true

require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
  include Rack::Test::Methods

  MUTATE_PARAMS = {
    name: 'String',
    from: 'Date',
    to: 'Date',
    housing_offer_id: ['offer1', 'offer2'],
    entity_id: ['entity1', 'entity2'],
  }.freeze

  ACCEPTABLE_PARAMS = %w[
    name
    from
    to
    housing_offer_id
    entity_id
  ].freeze

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $regex
  ].freeze


  describe Api::Dmo::EntityReservationsController, type: :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:responsible) {create(:user)}
    let!(:entity1) {create(:entity, siret: '00000000000001', user: responsible )}
    let!(:housing_offer1) {create(:housing_offer)}
    let!(:housing_offer2) {create(:housing_offer)}
    let!(:entity_reservation1) {create(:entity_reservation, 
      entity_id: entity1.id.to_s, 
      housing_offer_id: housing_offer1.id.to_s,
      from: Date.today - 30.days,
      to: Date.today + 30.days,
      name: 'entity_reservation1'
    )}


    describe 'GET /entity_reservations' do
      before :each do
        authenticated_header(request, responsible)
        get 'index'
      end

      it 'returns status code 200' do
        expect(response).to have_http_status(200)
        expect(json['data'].map{ |ev| ev['id'] }).to include(entity_reservation1.id.to_s)
      end
    end

    describe 'POST /entity_reservations' do
      let!(:params) { 
        {
          data: {
            attributes: {
              entity_id: entity1.id.to_s,
              housing_offer_id: housing_offer1.id.to_s,
              from: Date.today - 30.days,
              to: Date.today + 30.days,
            }
          }
        }
      }
      describe 'Create entity_reservation without minimal valid datas' do
        it 'returns status code 422' do
          authenticated_header(request, responsible)
          params[:data][:attributes][:entity_id] = nil
          post 'create',params
          expect(response).to have_http_status(422)
        end
      end
      
      describe 'Create entity_reservation with minimal valid datas' do
        it 'returns status code 201' do
          authenticated_header(request, responsible)
          params[:data][:attributes][:housing_offer_id] = housing_offer2.id.to_s
          post 'create',params
          expect(response).to have_http_status(201)
          expect(json['data']['id']).not_to eq(entity_reservation1.id.to_s)
        end
      end

      describe 'update existing reservation' do 
        it 'returns status code 201' do
          authenticated_header(request, responsible)
          post 'create', params
          expect(response).to have_http_status(201)
          expect(json['data']['id']).to eq(entity_reservation1.id.to_s)
        end
      end 
    end

    describe 'PUT /entity_reservations/:if' do
      let!(:entity_reservation2) { 
        create(:entity_reservation, 
          entity_id: entity1.id.to_s, 
          housing_offer_id: housing_offer2.id.to_s,
          from: Date.today - 30.days,
          to: Date.today + 30.days,
          name: 'reservation2'
        )
      }
      let!(:params) { 
        {
          id: entity_reservation1.id.to_s,
          data: {
            attributes: {
              entity_id: entity1.id.to_s,
              housing_offer_id: housing_offer1.id.to_s,
              from: Date.today - 28.days,
              to: Date.today + 30.days,
            }
          }
        }
      }

      it 'update from' do
        authenticated_header(request, responsible)
        post 'update', params
        expect(response).to have_http_status(200)
        expect(json['data']['id']).to eq(entity_reservation1.id.to_s)
        expect(entity_reservation1.reload.from).to eq(Date.today - 28.days)
      end 

      it 'when changing housing_offer_id' do 
        authenticated_header(request, responsible)
        params[:data][:attributes][:housing_offer_id] = housing_offer2.id.to_s

        post 'update', params
        expect(response).to have_http_status(200)
        expect(json['data']['id']).to eq(entity_reservation2.id.to_s)
        expect(entity_reservation2.reload.from).to eq(Date.today - 28.days)
      end 


    end 

  end
end